using System.Collections;
using System.Text.Json;
using EnumerableAsyncProcessor.Extensions;
using PkmnLib.Dynamic.Models;
using PkmnLib.Static.Species;
using PkmnLib.Tests.Integration.Models;

namespace PkmnLib.Tests.Integration;

public class IntegrationTestRunner
{
    public static IEnumerable<IntegrationTestModel> TestCases()
    {
        var files = Directory.GetFiles("Integration/Tests", "*.json");
        var serializerOptions = new JsonSerializerOptions
        {
            PropertyNameCaseInsensitive = true,
            AllowTrailingCommas = true,
            ReadCommentHandling = JsonCommentHandling.Skip,
        };
        foreach (var file in files)
        {
            var json = File.ReadAllText(file);
            var test = JsonSerializer.Deserialize<IntegrationTestModel>(json, serializerOptions)!;
            yield return test;
        }
    }

    [Test]
    [MethodDataSource(nameof(TestCases))]
    public async Task RunIntegrationTest(IntegrationTestModel test)
    {
        var library = LibraryHelpers.LoadLibrary();

        var parties = await test.BattleSetup.Parties.SelectAsync(async Task<IBattleParty> (x) =>
        {
            var party = new PokemonParty(6);
            for (var index = 0; index < x.Pokemon.Length; index++)
            {
                var pokemon = x.Pokemon[index];
                await Assert.That(library.StaticLibrary.Species.TryGet(pokemon.Species, out var species)).IsTrue();
                var mon = new PokemonImpl(library, species!, species!.GetDefaultForm(), new AbilityIndex
                    {
                        IsHidden = false,
                        Index = 0,
                    },
                    pokemon.Level, 0, Gender.Genderless, 0, "hardy");
                foreach (var move in pokemon.Moves)
                {
                    mon.LearnMove(move, MoveLearnMethod.Unknown, 255);
                }

                party.SwapInto(mon, index);
            }

            return new BattlePartyImpl(party, x.Indices.Select(y => new ResponsibleIndex(y[0], y[1])).ToArray());
        }).ProcessOneAtATime().GetResultsAsync();
        var battle = new BattleImpl(library, parties, test.BattleSetup.CanFlee, test.BattleSetup.NumberOfSides,
            test.BattleSetup.PositionsPerSide, randomSeed: test.BattleSetup.Seed);

        foreach (var action in test.Actions)
        {
            await action.Execute(battle);
        }
    }
}